@@ -22,12 +22,6 @@ def test_help(testdir):
         "  --benchmark-min-rounds=NUM",
         "                        Minimum rounds, even if total time would exceed",
         "                        `--max-time`. Default: 5",
-        "  --benchmark-sort=COL  Column to sort on. Can be one of: 'min', 'max', 'mean'",
-        "                        or 'stddev'. Default: 'min'",
-        "  --benchmark-group-by=LABEL",
-        "                        How to group tests. Can be one of: 'group', 'name',",
-        "                        'fullname', 'func', 'fullfunc' or 'param'. Default:",
-        "                        'group'",
         "  --benchmark-timer=FUNC",
         "                        Timer to use when measuring time. Default:*",
         "  --benchmark-calibration-precision=NUM",
@@ -45,7 +39,6 @@ def test_help(testdir):
         "  --benchmark-warmup-iterations=NUM",
         "                        Max number of iterations to run in the warmup phase.",
         "                        Default: 100000",
-        "  --benchmark-verbose   Dump diagnostic and progress information.",
         "  --benchmark-disable-gc",
         "                        Disable GC during benchmarks.",
         "  --benchmark-skip      Skip running any tests that contain benchmarks.",
@@ -54,31 +47,53 @@ def test_help(testdir):
         "                        Save the current run into 'STORAGE-",
         "                        PATH/counter_NAME.json'.",
         "  --benchmark-autosave  Autosave the current run into 'STORAGE-",
-        "                        PATH/counter_*.json",
+        "                        PATH/counter*.json",
         "  --benchmark-save-data",
         "                        Use this to make --benchmark-save and --benchmark-",
         "                        autosave include all the timing data, not just the",
         "                        stats.",
-        "  --benchmark-compare=[NUM]",
-        "                        Compare the current run against run NUM or the latest",
-        "                        saved run if unspecified.",
-        "  --benchmark-compare-fail=EXPR=[EXPR=...]",
+        "  --benchmark-json=PATH",
+        "                        Dump a JSON report into PATH. Note that this will",
+        "                        include the complete data (all the timings, not just",
+        "                        the stats).",
+        "  --benchmark-compare=[NUM|_ID]",
+        "                        Compare the current run against run NUM (or prefix of",
+        "                        _id in elasticsearch) or the latest saved run if",
+        "                        unspecified.",
+        "  --benchmark-compare-fail=EXPR?[[]EXPR?...[]]",
         "                        Fail test if performance regresses according to given",
         "                        EXPR (eg: min:5% or mean:0.001 for number of seconds).",
         "                        Can be used multiple times.",
-        "  --benchmark-storage=STORAGE-PATH",
-        "                        Specify a different path to store the runs (when",
-        "                        --benchmark-save or --benchmark-autosave are used).",
-        "                        Default: './.benchmarks/*'",
+        "  --benchmark-cprofile=COLUMN",
+        "                        If specified measure one run with cProfile and stores",
+        "                        10 top functions. Argument is a column to sort by.",
+        "                        Available columns: 'ncallls_recursion', 'ncalls',",
+        "                        'tottime', 'tottime_per', 'cumtime', 'cumtime_per',",
+        "                        'function_name'.",
+        "  --benchmark-storage=URI",
+        "                        Specify a path to store the runs as uri in form",
+        "                        file://path or elasticsearch+http[s]://host1,host2/[in",
+        "                        dex/doctype?project_name=Project] (when --benchmark-",
+        "                        save or --benchmark-autosave are used). For backwards",
+        "                        compatibility unexpected values are converted to",
+        "                        file://<value>. Default: 'file://./.benchmarks'.",
+        "  --benchmark-verbose   Dump diagnostic and progress information.",
+        "  --benchmark-sort=COL  Column to sort on. Can be one of: 'min', 'max',",
+        "                        'mean', 'stddev', 'name', 'fullname'. Default: 'min'",
+        "  --benchmark-group-by=LABEL",
+        "                        How to group tests. Can be one of: 'group', 'name',",
+        "                        'fullname', 'func', 'fullfunc', 'param' or",
+        "                        'param:NAME', where NAME is the name passed to",
+        "                        @pytest.parametrize. Default: 'group'",
+        "  --benchmark-columns=LABELS",
+        "                        Comma-separated list of columns to show in the result",
+        "                        table. Default: 'min, max, mean, stddev, median, iqr,",
+        "                        outliers, rounds, iterations'",
         "  --benchmark-histogram=[FILENAME-PREFIX]",
         "                        Plot graphs of min/max/avg/stddev over time in",
         "                        FILENAME-PREFIX-test_name.svg. If FILENAME-PREFIX",
         "                        contains slashes ('/') then directories will be",
         "                        created. Default: '*'",
-        "  --benchmark-json=PATH",
-        "                        Dump a JSON report into PATH. Note that this will",
-        "                        include the complete data (all the timings, not just",
-        "                        the stats).",
         "*",
     ])
 
@@ -160,6 +175,23 @@ def test_b(benchmark, foo):
     benchmark(int)
 '''
 
+GROUPING_PARAMS_TEST = '''
+import pytest
+
+@pytest.mark.parametrize("bar", ["bar1", "bar2"])
+@pytest.mark.parametrize("foo", ["foo1", "foo2"])
+@pytest.mark.benchmark(group="A")
+def test_a(benchmark, foo, bar):
+    benchmark(str)
+
+
+@pytest.mark.parametrize("bar", ["bar1", "bar2"])
+@pytest.mark.parametrize("foo", ["foo1", "foo2"])
+@pytest.mark.benchmark(group="B")
+def test_b(benchmark, foo, bar):
+    benchmark(int)
+'''
+
 
 def test_group_by_name(testdir):
     test_x = testdir.makepyfile(test_x=GROUPING_TEST)
@@ -259,12 +291,13 @@ def test_group_by_fullfunc(testdir):
         'test_b[[]*[]]         *',
         '------------------*',
         '',
-        '(*) Outliers: 1 Standard Deviation from M*',
+        'Legend:',
+        '  Outliers: 1 Standard Deviation from M*',
         '============* 8 passed* seconds ============*',
     ])
 
 
-def test_group_by_param(testdir):
+def test_group_by_param_all(testdir):
     test_x = testdir.makepyfile(test_x=GROUPING_TEST)
     test_y = testdir.makepyfile(test_y=GROUPING_TEST)
     result = testdir.runpytest('--benchmark-max-time=0.0000001', '--benchmark-group-by', 'param', test_x, test_y)
@@ -288,7 +321,84 @@ def test_group_by_param(testdir):
         'test_*[[]1[]]         *',
         '------------------*',
         '',
-        '(*) Outliers: 1 Standard Deviation from Mean; 1.5 IQR (InterQuartile Range) from 1st Quartile and 3rd '
+        'Legend:',
+        '  Outliers: 1 Standard Deviation from Mean; 1.5 IQR (InterQuartile Range) from 1st Quartile and 3rd '
+        'Quartile.',
+        '============* 8 passed* seconds ============*',
+    ])
+
+
+def test_group_by_param_select(testdir):
+    test_x = testdir.makepyfile(test_x=GROUPING_PARAMS_TEST)
+    result = testdir.runpytest('--benchmark-max-time=0.0000001',
+                               '--benchmark-group-by', 'param:foo',
+                               '--benchmark-sort', 'fullname',
+                               test_x)
+    result.stdout.fnmatch_lines([
+        '*', '*', '*', '*', '*',
+        "* benchmark 'foo=foo1': 4 tests *",
+        'Name (time in ?s)  *',
+        '-------------------*',
+        'test_a[[]foo1-bar1[]]    *',
+        'test_a[[]foo1-bar2[]]    *',
+        'test_b[[]foo1-bar1[]]    *',
+        'test_b[[]foo1-bar2[]]    *',
+        '-------------------*',
+        '',
+        "* benchmark 'foo=foo2': 4 tests *",
+        'Name (time in ?s) *',
+        '------------------*',
+        'test_a[[]foo2-bar1[]]    *',
+        'test_a[[]foo2-bar2[]]    *',
+        'test_b[[]foo2-bar1[]]    *',
+        'test_b[[]foo2-bar2[]]    *',
+        '------------------*',
+        '',
+        'Legend:',
+        '  Outliers: 1 Standard Deviation from Mean; 1.5 IQR (InterQuartile Range) from 1st Quartile and 3rd '
+        'Quartile.',
+        '============* 8 passed* seconds ============*',
+    ])
+
+
+def test_group_by_param_select_multiple(testdir):
+    test_x = testdir.makepyfile(test_x=GROUPING_PARAMS_TEST)
+    result = testdir.runpytest('--benchmark-max-time=0.0000001',
+                               '--benchmark-group-by', 'param:foo,param:bar',
+                               '--benchmark-sort', 'fullname',
+                               test_x)
+    result.stdout.fnmatch_lines([
+        '*', '*', '*', '*', '*',
+        "* benchmark 'foo=foo1 bar=bar1': 2 tests *",
+        'Name (time in ?s)  *',
+        '-------------------*',
+        'test_a[[]foo1-bar1[]]    *',
+        'test_b[[]foo1-bar1[]]    *',
+        '-------------------*',
+        '',
+        "* benchmark 'foo=foo1 bar=bar2': 2 tests *",
+        'Name (time in ?s)  *',
+        '-------------------*',
+        'test_a[[]foo1-bar2[]]    *',
+        'test_b[[]foo1-bar2[]]    *',
+        '-------------------*',
+        '',
+        "* benchmark 'foo=foo2 bar=bar1': 2 tests *",
+        'Name (time in ?s) *',
+        '------------------*',
+        'test_a[[]foo2-bar1[]]    *',
+        'test_b[[]foo2-bar1[]]    *',
+        '-------------------*',
+        '',
+        "* benchmark 'foo=foo2 bar=bar2': 2 tests *",
+        'Name (time in ?s)  *',
+        '-------------------*',
+        'test_a[[]foo2-bar2[]]    *',
+        'test_b[[]foo2-bar2[]]    *',
+        '------------------*',
+        '',
+        'Legend:',
+        '  Outliers: 1 Standard Deviation from Mean; 1.5 IQR (InterQuartile Range) from 1st Quartile and 3rd '
         'Quartile.',
         '============* 8 passed* seconds ============*',
     ])
@@ -447,12 +557,12 @@ def test_compare(testdir):
     result = testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules', '--benchmark-compare=0001',
                                '--benchmark-compare-fail=min:0.1', test)
     result.stderr.fnmatch_lines([
-        "Comparing against benchmark 0001_unversioned_*.json:",
+        "Comparing against benchmarks from: *0001_unversioned_*.json",
     ])
     result = testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules', '--benchmark-compare=0001',
                                '--benchmark-compare-fail=min:1%', test)
     result.stderr.fnmatch_lines([
-        "Comparing against benchmark 0001_unversioned_*.json:",
+        "Comparing against benchmarks from: *0001_unversioned_*.json",
     ])
 
 
@@ -462,12 +572,12 @@ def test_compare_last(testdir):
     result = testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules', '--benchmark-compare',
                                '--benchmark-compare-fail=min:0.1', test)
     result.stderr.fnmatch_lines([
-        "Comparing against benchmark 0001_unversioned_*.json:",
+        "Comparing against benchmarks from: *0001_unversioned_*.json",
     ])
     result = testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules', '--benchmark-compare',
                                '--benchmark-compare-fail=min:1%', test)
     result.stderr.fnmatch_lines([
-        "Comparing against benchmark 0001_unversioned_*.json:",
+        "Comparing against benchmarks from: *0001_unversioned_*.json",
     ])
 
 
@@ -477,7 +587,7 @@ def test_compare_non_existing(testdir):
     result = testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules', '--benchmark-compare=0002', '-rw',
                                test)
     result.stdout.fnmatch_lines([
-        "WBENCHMARK-C1 * Can't compare. No benchmark files matched '0002'",
+        "WBENCHMARK-C1 * Can't compare. No benchmark files * '0002'.",
     ])
 
 
@@ -487,7 +597,7 @@ def test_compare_non_existing_verbose(testdir):
     result = testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules', '--benchmark-compare=0002',
                                test, '--benchmark-verbose')
     result.stderr.fnmatch_lines([
-        " WARNING: Can't compare. No benchmark files matched '0002'",
+        " WARNING: Can't compare. No benchmark files * '0002'.",
     ])
 
 
@@ -496,8 +606,8 @@ def test_compare_no_files(testdir):
     result = testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules', '-rw',
                                test, '--benchmark-compare')
     result.stdout.fnmatch_lines([
-         "WBENCHMARK-C3 * Can't compare. No benchmark files in '*'. Expected files matching *.json."
-         " Can't load the previous benchmark."
+        "WBENCHMARK-C2 * Can't compare. No benchmark files in '*'."
+        " Can't load the previous benchmark."
     ])
 
 
@@ -506,7 +616,7 @@ def test_compare_no_files_verbose(testdir):
     result = testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules',
                                test, '--benchmark-compare', '--benchmark-verbose')
     result.stderr.fnmatch_lines([
-        " WARNING: Can't compare. No benchmark files in '*'. Expected files matching *.json."
+        " WARNING: Can't compare. No benchmark files in '*'."
         " Can't load the previous benchmark."
     ])
 
@@ -516,8 +626,7 @@ def test_compare_no_files_match(testdir):
     result = testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules', '-rw',
                                test, '--benchmark-compare=1')
     result.stdout.fnmatch_lines([
-        "WBENCHMARK-C4 * Can't compare. No benchmark files in '*'. Expected files matching *.json."
-         " Can't match anything to '1'."
+        "WBENCHMARK-C1 * Can't compare. No benchmark files in '*' match '1'."
     ])
 
 
@@ -526,34 +635,7 @@ def test_compare_no_files_match_verbose(testdir):
     result = testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules',
                                test, '--benchmark-compare=1', '--benchmark-verbose')
     result.stderr.fnmatch_lines([
-        " WARNING: Can't compare. No benchmark files in '*'. Expected files matching *.json."
-        " Can't match anything to '1'."
-    ])
-
-
-def test_compare_too_many(testdir):
-    test = testdir.makepyfile(SIMPLE_TEST)
-    testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules', '--benchmark-autosave', test)
-    testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules', '--benchmark-autosave', test)
-    result = testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules', '--benchmark-compare=0', '-rw',
-                               test)
-    result.stdout.fnmatch_lines([
-        "WBENCHMARK-C2 * Can't compare. Too many benchmark files matched '0':",
-        ' - *0001_unversioned_*.json',
-        ' - *0002_unversioned_*.json',
-    ])
-
-
-def test_compare_too_many_verbose(testdir):
-    test = testdir.makepyfile(SIMPLE_TEST)
-    testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules', '--benchmark-autosave', test)
-    testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules', '--benchmark-autosave', test)
-    result = testdir.runpytest('--benchmark-max-time=0.0000001', '--doctest-modules', '--benchmark-compare=0',
-                               '--benchmark-verbose', test)
-    result.stderr.fnmatch_lines([
-        " WARNING: Can't compare. Too many benchmark files matched '0':",
-        ' - *0001_unversioned_*.json',
-        ' - *0002_unversioned_*.json',
+        " WARNING: Can't compare. No benchmark files in '*' match '1'."
     ])
 
 
@@ -575,22 +657,37 @@ def test_save(testdir):
     result = testdir.runpytest('--doctest-modules', '--benchmark-save=foobar',
                                '--benchmark-max-time=0.0000001', test)
     result.stderr.fnmatch_lines([
-        "Saved benchmark data in *",
+        "Saved benchmark data in: *",
     ])
     json.loads(testdir.tmpdir.join('.benchmarks').listdir()[0].join('0001_foobar.json').read())
 
 
+def test_save_extra_info(testdir):
+    test = testdir.makepyfile("""
+    def test_extra(benchmark):
+        benchmark.extra_info['foo'] = 'bar'
+        benchmark(lambda: None)
+    """)
+    result = testdir.runpytest('--doctest-modules', '--benchmark-save=foobar',
+                               '--benchmark-max-time=0.0000001', test)
+    result.stderr.fnmatch_lines([
+        "Saved benchmark data in: *",
+    ])
+    info = json.loads(testdir.tmpdir.join('.benchmarks').listdir()[0].join('0001_foobar.json').read())
+    bench_info = info['benchmarks'][0]
+    assert bench_info['name'] == 'test_extra'
+    assert bench_info['extra_info'] == {'foo': 'bar'}
+
+
 def test_histogram(testdir):
     test = testdir.makepyfile(SIMPLE_TEST)
     result = testdir.runpytest('--doctest-modules', '--benchmark-histogram=foobar',
                                '--benchmark-max-time=0.0000001', test)
     result.stderr.fnmatch_lines([
-        "Generated histogram *foobar-test_histogram.py_test_fast.svg",
-        "Generated histogram *foobar-test_histogram.py_test_slow.svg",
+        "Generated histogram: *foobar.svg",
     ])
     assert [f.basename for f in testdir.tmpdir.listdir("*.svg", sort=True)] == [
-        'foobar-test_histogram.py_test_fast.svg',
-        'foobar-test_histogram.py_test_slow.svg',
+        'foobar.svg',
     ]
 
 
@@ -599,7 +696,7 @@ def test_autosave(testdir):
     result = testdir.runpytest('--doctest-modules', '--benchmark-autosave',
                                '--benchmark-max-time=0.0000001', test)
     result.stderr.fnmatch_lines([
-        "Saved benchmark data in *",
+        "Saved benchmark data in: *",
     ])
     json.loads(testdir.tmpdir.join('.benchmarks').listdir()[0].listdir('0001_*.json')[0].read())
 
@@ -676,8 +773,8 @@ def test_bogus_sort(testdir):
     result = testdir.runpytest('--benchmark-sort=bogus', test)
     result.stderr.fnmatch_lines([
         "usage: py* [[]options[]] [[]file_or_dir[]] [[]file_or_dir[]] [[]...[]]",
-        "py*: error: argument --benchmark-sort: Unacceptable value: 'bogus'. Value for --benchmark-sort must be one of: 'min', 'max', 'mean' or 'stddev'."
-
+        "py*: error: argument --benchmark-sort: Unacceptable value: 'bogus'. Value for --benchmark-sort must be one "
+        "of: 'min', 'max', 'mean', 'stddev', 'name', 'fullname'."
     ])
 
 
@@ -703,6 +800,26 @@ def test_xdist_verbose(testdir):
     ])
 
 
+def test_cprofile(testdir):
+    test = testdir.makepyfile(SIMPLE_TEST)
+    result = testdir.runpytest('--benchmark-cprofile=cumtime', test)
+    result.stdout.fnmatch_lines([
+        "============*=========== cProfile information ============*===========",
+        "Time in s",
+        "test_cprofile.py::test_fast",
+        "ncalls	tottime	percall	cumtime	percall	filename:lineno(function)",
+        # "1	0.0000	0.0000	0.0001	0.0001	test_cprofile0/test_cprofile.py:9(result)",
+        # "1	0.0001	0.0001	0.0001	0.0001	~:0(<built-in method time.sleep>)",
+        # "1	0.0000	0.0000	0.0000	0.0000	~:0(<method 'disable' of '_lsprof.Profiler' objects>)",
+        "",
+        "test_cprofile.py::test_slow",
+        "ncalls	tottime	percall	cumtime	percall	filename:lineno(function)",
+        # "1	0.0000	0.0000	0.1002	0.1002	test_cprofile0/test_cprofile.py:15(<lambda>)",
+        # "1	0.1002	0.1002	0.1002	0.1002	~:0(<built-in method time.sleep>)",
+        # "1	0.0000	0.0000	0.0000	0.0000	~:0(<method 'disable' of '_lsprof.Profiler' objects>)",
+    ])
+
+
 def test_abort_broken(testdir):
     """
     Test that we don't benchmark code that raises exceptions.
@@ -781,7 +898,7 @@ def test_ok(benchmark, bad_fixture):
         "*====== FAILURES ======*",
         "*______ test_bad ______*",
 
-        "benchmark = <pytest_benchmark.plugin.BenchmarkFixture object at *>",
+        "benchmark = <pytest_benchmark.*.BenchmarkFixture object at *>",
 
         "    def test_bad(benchmark):",
         ">       @benchmark",
@@ -789,12 +906,7 @@ def test_ok(benchmark, bad_fixture):
 
         "test_abort_broken.py:*",
         "_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _*",
-        "*pytest_benchmark/plugin.py:*: in __call__",
-        "    duration, iterations, loops_range = self._calibrate_timer(runner)",
-        "*pytest_benchmark/plugin.py:*: in _calibrate_timer",
-        "    duration = runner(loops_range)",
-        "*pytest_benchmark/plugin.py:*: in runner",
-        "    *",
+        "*",
         "_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _*",
 
         "    @benchmark",
@@ -805,7 +917,7 @@ def test_ok(benchmark, bad_fixture):
         "test_abort_broken.py:11: Exception",
         "*______ test_bad2 ______*",
 
-        "benchmark = <pytest_benchmark.plugin.BenchmarkFixture object at *>",
+        "benchmark = <pytest_benchmark.*.BenchmarkFixture object at *>",
 
         "    def test_bad2(benchmark):",
         "        @benchmark",
@@ -815,6 +927,9 @@ def test_ok(benchmark, bad_fixture):
         "E       assert 1 == 0",
 
         "test_abort_broken.py:18: AssertionError",
+    ])
+
+    result.stdout.fnmatch_lines([
         "* benchmark: 1 tests *",
         "Name (time in ?s) * Min * Max * Mean * StdDev * Rounds * Iterations",
         "------*",
@@ -920,7 +1035,7 @@ def test_mark_selection(testdir):
         "------*",
         "test_xfast       *",
         "------*",
-        "*====== 4 tests deselected by \"-m 'benchmark'\" ======*",
+        "*====== 4 tests deselected* ======*",
         "*====== 1 passed, 4 deselected* seconds ======*",
     ])
 
@@ -945,3 +1060,15 @@ def test_only_benchmarks(testdir):
         "------*",
         "*====== 4 passed, 1 skipped* seconds ======*",
     ])
+
+
+def test_columns(testdir):
+    test = testdir.makepyfile(SIMPLE_TEST)
+    result = testdir.runpytest('--doctest-modules', '--benchmark-columns=max,iterations,min', test)
+    result.stdout.fnmatch_lines([
+        "*collected 3 items",
+        "test_columns.py ...",
+        "* benchmark: 2 tests *",
+        "Name (time in ?s) * Max * Iterations * Min *",
+        "------*",
+    ])
